﻿var treeNodeClasses = {
    error: 'error',
    warning: 'warning',
    notInBasket: 'notInBasket',
    notSelected: 'notSelected',
    noClasses: ''
};

var temporaryBasketTree;

function mapItemsRecursively(item) {
    var newNode = {
        title: item.name,
        key: item.id.id,
        icon: item.iconPath,
        expanded: true,
        canSelect: item.canSelect,
        children: [],
        validationStatus: null,
        valid: null
    };

    if (item.canSelect === false) {
        newNode.hideCheckbox = true;
        newNode.extraClasses = treeNodeClasses.notInBasket;
        newNode.unselectable = true;
    } else {
        newNode.selected = true;
    }

    if (item.validationResult) {
        newNode.validationStatus = item.validationResult.errorCode;
        newNode.valid = item.validationResult.valid;

        if (!item.validationResult.valid) {
            newNode.extraClasses = treeNodeClasses.error;

            if (item.validationResult.errorCode === window.validationErrors.disallowedByConfiguration) {
                newNode.selected = false;
                newNode.unselectable = true;
            }
        } else if (item.validationResult.errorCode === window.validationErrors.noTranslatableContent) {
            item.extraClasses = treeNodeClasses.warning;
        }
    }

    if (item.children && item.children.length > 0) {
        $j.each(item.children, function () {
            newNode.children.push(mapItemsRecursively(this));
        });
    }

    return newNode;
};

function prepareItemModel(item) {
    if (item.validationResult) {
        item.validationStatus = item.validationResult.errorCode;
        item.valid = item.validationResult.valid;
    }
}

function getDocument(itemId, selected) {
    $j.ajax({
        type: 'POST',
        contentType: 'application/json; charset=utf-8',
        cache: false,
        async: true,
        url: '../Services/ProjectWebservice.asmx/GetTemporaryBasketItem',
        data: JSON.stringify({ itemId: itemId }),
        dataType: 'text',
        success: function (data) {
            var itemModel = $j.parseJSON($j.parseJSON(data.match(/({"d.*)/)[1]).d); //use only " in other cases regex willn't work
            prepareItemModel(itemModel);
            var output = $j.render.ViewDocumentInfoTmpl(itemModel, {
                selected: selected,
                validationErrors: window.validationErrors
            });
            $j('#divConfirmTransaction').html(output);
        },
        error: function () {
            console.log('Error occured during document info request');
        }
    });
};

function showDefaultSummary() {
    $j('#divConfirmTransaction').html($j.render.defaultviewTmpl());
};

function initFancyTree(data) {
    $j('#tree').fancytree({
        source: data,
        checkbox: true,
        clickFolderMode: 3,
        includeRoot: true,
        activate: function (event, data) {
            onNodeActivated(data.node);
        },
        select: function (event, data) {
            onNodeSelected(data.node);
        }
    });

    temporaryBasketTree = $j('#tree').fancytree('getTree');

    countAndValidateNoOfItems();
};

function onNodeActivated(node) {
    if (node.data.canSelect) {
        getDocument(node.key, node.isSelected());
    } else {
        showDefaultSummary();
    }
};

function onNodeSelected(node) {

    if (node.data.validationStatus !== validationErrors.duplicatedMultirelational) {
        if (!node.selected) {
            node.extraClasses += ' ' + treeNodeClasses.notSelected;
        } else {
            node.extraClasses = node.extraClasses.replace(' ' + treeNodeClasses.notSelected, '');
        }
    } else {
        if (!node.selected) {
            node.extraClasses = treeNodeClasses.noClasses;
        } else {
            node.extraClasses = treeNodeClasses.error;
        }
    }

    node.render();

    countAndValidateNoOfItems();
    if (node.isActive()) {
        getDocument(node.key, node.isSelected());
    }
};

function calculateSelectedItems() {
    var duplicatedItems = 0;
    var newItemsCount = 0;
    var calculationFunction = function (node) {
        if (node.selected && node.data.validationStatus !== window.validationErrors.disallowedByConfiguration) {
            if (node.data.validationStatus === window.validationErrors.duplicatedItem || node.data.validationStatus === window.validationErrors.duplicatedMultirelational) {
                duplicatedItems++;
            }
            else {
                newItemsCount++;
            }
        }
    };

    temporaryBasketTree.visit(calculationFunction);

    return { newItems: newItemsCount, duplicatedItems: duplicatedItems };
};

function countDeselectedItems() {
    var deselectedItems = 0;
    temporaryBasketTree.visit(function (node) {
        if (!node.selected && node.data.canSelect) {
            deselectedItems++;
        }
    });
    return deselectedItems;
};

function removeDocuments(documentsId) {
    $j.ajax({
        type: 'POST',
        contentType: 'application/json; charset=utf-8',
        cache: false,
        async: true,
        url: '../Services/ProjectWebservice.asmx/RemoveDocumentsFromTemporaryBasket',
        data: JSON.stringify({ documentIds: documentsId }),
        dataType: 'json',
        success: function () {
            commitTemporaryBasket();
        },
        error: function () {
            console.log('Error occured during documents remove');
        }
    });
};

function commitTemporaryBasket() {
    $j.ajax({
        type: 'POST',
        contentType: 'application/json; charset=utf-8',
        cache: false,
        async: true,
        url: '../Services/ProjectWebservice.asmx/GetBasketBriefDetails',
        data: JSON.stringify({ includingTemporaryBasket: true }),
        dataType: 'json',
        success: function (data) {
            $j('#' + window.okButtonId).click();
            var basketBriefDetails = $j.parseJSON(data.d);
            if (window.parent && window.parent.gadget) {
                window.parent.gadget.updatePanels(basketBriefDetails.itemsCount, basketBriefDetails.basketLanguage);
                window.parent.gadget.closeShowTransaction();
            }
        },
        error: function () {
            console.log('Error occured during temporary basket commit');
        }
    });
};

function initializeModalWindow() {
    $j('#noOfItemsValidateResultDiv').hide();

    showDefaultSummary();

    $j.ajax({
        type: 'POST',
        contentType: 'application/json; charset=utf-8',
        cache: false,
        async: true,
        url: '../Services/ProjectWebservice.asmx/GetLatestTransactionFancytree',
        dataType: 'text',

        success: function (data) {
            var treeModel = $j.parseJSON(data.match(/({"rootItem.*){"d":null}/)[1]);
            if (treeModel.rootItem) {
                initFancyTree(mapItemsRecursively(treeModel.rootItem));
            } else {
                $j('#divConfirmTransaction').html($j.render.emptyBasketViewTmpl());
            }
        },
        error: function () {
            console.log('Error occured during modal window initialization');
        }
    });
};

function countAndValidateNoOfItems() {
    $j('#btnOk').removeAttr('disabled');
    $j('#noOfItemsValidateResultDiv').hide();

    var selectedItemCount = calculateSelectedItems();
    var newItemCount = selectedItemCount.newItems;
    var total = newItemCount + alreadyAdded;

    if ((selectedItemCount.duplicatedItems + newItemCount) === 0) {
        $j('#btnOk').attr('disabled', 'disabled');
        return;
    }

    if (total > maxNoOfItems) {
        var canBeAddedCount = maxNoOfItems - alreadyAdded;
        var errMsg = temporaryBasketTranslatedStrings.tooManyItemsInBasket;
        var pluralizeNewItemCount = newItemCount !== 1 ? 's' : '';
        var pluralizeCanBeAddedCount = canBeAddedCount !== 1 ? 's' : '';
        errMsg = stringFormat(errMsg, newItemCount, pluralizeNewItemCount, canBeAddedCount, pluralizeCanBeAddedCount);
        $j('#noOfItemsValidateResult').html(errMsg);
        $j('#noOfItemsValidateResultDiv').show();
        $j('#btnOk').attr('disabled', 'disabled');
    }
};

function okOnClick() {
    $j('#btnOk').attr('disabled', 'disabled');
    $j('#btnOk').prop('value', temporaryBasketTranslatedStrings.updatingButton);
    $j('#btnOk').parent().removeClass('col-xs-1 col-xs-offset-10').addClass('col-xs-2 col-xs-offset-9');

    if (countDeselectedItems() > 0) {
        var ids = [];
        temporaryBasketTree.visit(function (node) {
            if (node.data.canSelect && !node.selected) {
                ids.push(node.key);
            }
        });
        if (ids.length > 0) {
            removeDocuments(ids);
        }
    } else {
        commitTemporaryBasket();
    }
};

function onCancelClick() {
    if (window.parent && window.parent.gadget) {
        window.parent.gadget.closeShowTransaction();
    }
};